@@ -1,6 +1,6 @@
 ! RUN: %python %S/test_errors.py %s %flang_fc1
 subroutine test1
-  !ERROR: Generic interface 'foo' has both a function and a subroutine
+  !WARNING: Generic interface 'foo' has both a function and a subroutine [-Wsubroutine-and-function-specifics]
   interface foo
     subroutine s1(x)
     end subroutine
@@ -12,19 +12,19 @@ subroutine test1
 end subroutine
 
 subroutine test2
-  !ERROR: Generic interface 'foo' has both a function and a subroutine
+  !WARNING: Generic interface 'foo' has both a function and a subroutine [-Wsubroutine-and-function-specifics]
   interface foo
-    function f1(x)
+    function t2f1(x)
     end function
     subroutine s()
     end subroutine
-    function f2(x, y)
+    function t2f2(x, y)
     end function
   end interface
 end subroutine
 
 module test3
-  !ERROR: Generic interface 'foo' has both a function and a subroutine
+  !WARNING: Generic interface 'foo' has both a function and a subroutine [-Wsubroutine-and-function-specifics]
   interface foo
     module procedure s
     module procedure f
@@ -39,7 +39,7 @@ end module
 subroutine test4
   type foo
   end type
-  !ERROR: Generic interface 'foo' may only contain functions due to derived type with same name
+  !WARNING: Generic interface 'foo' should only contain functions due to derived type with same name [-Wsubroutine-and-function-specifics]
   interface foo
     subroutine s()
     end subroutine
@@ -48,13 +48,13 @@ end subroutine
 
 subroutine test5
   interface foo
-    function f1()
+    function t5f1()
     end function
   end interface
   interface bar
-    subroutine s1()
+    subroutine t5s1()
     end subroutine
-    subroutine s2(x)
+    subroutine t5s2(x)
     end subroutine
   end interface
   !ERROR: Cannot call function 'foo' like a subroutine
